/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: serial.c
*/

#include "core.h"



// Buffering for controller to host data
static unsigned char host_tx_buff[MAX_HOST_TX];
static int host_tx_tail = 0;
static int host_tx_head = 0;
static int host_tx_pending = 0;


/* Compute checksum of packet */
unsigned int get_checksum(unsigned char *buff, unsigned int len)
{
	unsigned char ck = 0;
	
	while(len)
	{
		ck += buff[--len];
	}

	return(ck);
}
/* End of get_checksum */


/* Send data to host */
void serial_tx( unsigned char chan, unsigned char d )
{
	// Old polling method
	#if !BUFFER_HOST_TX

		#ifdef U1HOST
		while (U1STAbits.UTXBF);
		U1TXREG = d;
		#endif

		#ifdef U2HOST
		while (U2STAbits.UTXBF);
		U2TXREG = d;
		#endif

		return;
	#endif



	while(host_tx_pending > MAX_HOST_TX-2);

	interrupt_disable();


	// Add data to the buffer
	host_tx_buff[host_tx_head++] = d;

	// Another byte is now pending
	host_tx_pending++;
	
	// Wrap buffer
	if (host_tx_head  >= MAX_HOST_TX)
		host_tx_head = 0;


	// Force tx interrupt if the buffer was empty
	if (host_tx_pending == 1)
		IFS0bits.U1TXIF = 1; 


	interrupt_enable();
	
	return;


}
/* End of host send */


/* Send to host complete, send next byte if any */
inline void host_tx_complete( void )
{
	// Buffer empty, nothing to send
	if (host_tx_pending == 0) 
	{
		return;
	}
	
	// Refill the buffer
	//while (host_tx_pending && !U1STAbits.UTXBF)
	while (host_tx_pending && !U1STAbits.UTXBF)
	{
		// Send next byte
		U1TXREG = host_tx_buff[host_tx_tail++];		// Remove byte from buffer and send it

		host_tx_pending--;

		// Wrap buffer
		if (host_tx_tail  >= MAX_HOST_TX)
			host_tx_tail = 0;

	}
}
/* End of host_tx_complete */




/* UART1 Serial RX Interrupt */
//  Data from the host
void __attribute__((__interrupt__, auto_psv)) _U1RXInterrupt (void)
{
	IFS0bits.U1RXIF = 0;

	#ifdef U1HOST
	process_host_rx(U1RXREG & 0xff);
	#endif

}
/* End of _U1RXInterrupt */



/* UART1 Serial TX Interrupt */
void __attribute__((__interrupt__, auto_psv)) _U1TXInterrupt (void)
{	
	IFS0bits.U1TXIF = 0;			// Clear transmit interrupt flag
//	host_tx_complete();
}
/* End of _U1TXInterrupt */



/* UART2 Serial RX Interrupt */
void __attribute__((__interrupt__, auto_psv)) _U2RXInterrupt (void)
{
	IFS1bits.U2RXIF = 0;

	#ifdef U2HOST
	process_host_rx(U2RXREG & 0xff);
	#endif

}
/* End of _U1RXInterrupt */



/* UART1 Serial TX Interrupt */
void __attribute__((__interrupt__, auto_psv)) _U2TXInterrupt (void)
{
	IFS1bits.U2TXIF = 0;			// Clear transmit interrupt flag
//	host_tx_complete();
}
/* End of _U1TXInterrupt */



/* Reset systems for a serial timeout event */
void serial_timeout ( void )
{
	controller_reset();

	host_tx_tail = 0;
	host_tx_head = 0;
	host_tx_pending = 0;

	comm_reset();


	if (U2STAbits.OERR) fault |= FAULT_OE;
	if (U2STAbits.FERR) fault |= FAULT_FE;



	// Clear error flags
	U1STAbits.OERR = 0;
	U1STAbits.FERR = 0;

	U2STAbits.OERR = 0;
	U2STAbits.FERR = 0;
}
/* End of serial timeout */


/* Update serial system */
void serial_timer( void )
{
/*
	// Clear error flags
	U1STAbits.OERR = 0;
	U1STAbits.FERR = 0;

	U2STAbits.OERR = 0;
	U2STAbits.FERR = 0;*/
}
/* End of serial_timer */


/* Setup serial */
void serial_init( void )
{
	// Host connected to first uart 
	#ifdef U1HOST
	U1BRG = (((FCY / HOST_BAUD) / 16) - 1);		// Create and set baud counter

//	IPC2bits.U1TXIP = 7;
	//IPC2bits.U1RXIP = 7;

	IEC0bits.U1RXIE = 1;
	IEC0bits.U1TXIE = 1;

//	U1STAbits.UTXISEL = 1;

	IFS0bits.U1RXIF = 0;	// Clear interrut flag

	U1MODEbits.UARTEN = 1;
	U1STAbits.UTXEN = 1;
	#endif


	// Host connected to second uart
	#ifdef U2HOST
	U2BRG = (((FCY / HOST_BAUD) / 16) - 1);		// Create and set baud counter


	U2MODEbits.UARTEN = 1;

	IPC6bits.U2TXIP = 7;
	IPC6bits.U2RXIP = 7;


	U2STAbits.UTXISEL = 1;

	IFS1bits.U2RXIF = 0;	// Clear interrut flag
	IEC1bits.U2RXIE = 1;
	IEC1bits.U2TXIE = 1;

	U2STAbits.UTXEN = 1;

	#endif

	// Setup Timer
	timer_register(serial_timer);
}
/* End of serial_init */


