/*
    Robot Controller Client

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: world.h
*/

#ifndef WORLD_H
#define WORLD_H

#include <windows.h>

#include "types.h"
#include "utility.h"
#include "main.h"
#include  "robot.h"
#include "map.h"

// Size of gradient display grid
#define NUM_Y_GRAD 75
#define NUM_X_GRAD 75

#define MAX_ROBOT   100


#define  MAX_DIST 10000


// Linked list type for impacts
typedef struct impact_type
{
    double x, y;
    impact_type *next;
};


// Linked list type for markers
typedef struct marker_type
{
    double x, y;
    marker_type *next;
};


// Linked list type for walls
typedef struct wall_type
{
    double x, y;
    double x_size, y_size;
    wall_type *next;
};




// World structure
class world_class
{

  public:

    world_class( main_class *m);
    ~world_class( void );

    void load_robots( void );


    void draw( HDC dc );              // Display world
    void update( void );              // Update world

    int get_screen_x( double x );           // Convert world to screen pos
    int get_screen_y( double y );

    double get_world_x( unsigned int x);    // Convert Screen to world pos
    double get_world_y( unsigned int y);

  //  int get_map_x(double x);                // Convert world to map pos
//    int get_map_y(double y);


    // Determine bounding rectangle for world scace coords
    void get_bounding_rect(double x, double y, double x_size, double y_size, RECT &r);


    // Wall functions
    void add_wall(double x, double y, double y_size, double x_size);
    void free_walls( void );
    void draw_walls( HDC dc );
    bool check_wall_point( double x, double y );

    // Set target position
    void set_target(double x, double y);

    // Marker functions
    void draw_markers( HDC dc );
    void free_markers( void );
    void add_marker(double x, double y);

    // Impact functions
    void draw_impacts( HDC dc );
    void free_impacts( void );
    void add_impact(double x, double y);

    // Waypoint functions
//    void free_waypoints( void);
//    void draw_waypoints(HDC dc);
//    void add_waypoint(double x, double y, double dir);

    // Gradient functions
    void draw_grad(HDC dc);
    void compute_grad_point(robot_class *r, double x, double y, double &dir, double &mag);

    void save( void );
    void load( void );

    //void compute_waypoint_vector(double x, double y, point &v);
    void compute_impact_vector(double x, double y, point &v);
    
    
    void draw_arrow(HDC dc, double x, double y, double dir, double size);
    void draw_line(HDC dc, line l);
    void draw_sensor(HDC dc, double x, double y, double dir, double dist, double size, COLORREF c);
      
    
    void robot_parse(char *line);
    static void robot_parse_wrapper(void *w, char *line);


    void world_draw_map_grid(robot_class *robot,  HDC dc );        
        
        
    double check_vision(double x, double y, double dir, bool &found, point &p) ;
    bool check_sensor(double x, double y, double dir, double dist)   ;    
          

    // Accessors
    void toggle_walls( void ) { show_walls = !show_walls; };
        
    double get_wall0_x( void ) { return walls->x; };
    double get_wall0_y( void ) { return walls->y; };
    double set_wall0_size( double xs, double ys) { walls->x_size = xs; walls->y_size = ys; };
         
    bool wall_present( void ) { return walls != NULL; };   
    
    
    void  random_target( void );
    
    
    double get_x_size( void ) { return x_size; };
    double get_y_size( void ) { return y_size; };
    
    
    double target_dist(double x, double t);

    
    impact_type * get_impacts( void ) { return impacts; };
    
    double get_target_x( void ) { return target_x; };
    double get_target_y( void ) { return target_y; };
    
    
    void toggle_show_markers( void ) { show_markers = !show_markers; };
    void toggle_show_impacts( void ) { show_impacts = !show_impacts; };

    void toggle_show_map( void ) { show_map = !show_map; };
    void toggle_show_grad( void ) { show_grad = !show_grad; };


    void world_class::draw_target(HDC dc, double x, double y);
    
    
    
    
  private:
      
  // Size of world
    double x_size, y_size;           
    
    // Wall data    
    wall_type *walls;
    unsigned int num_walls;


        
    // Marker data
    marker_type *markers;



    // Impact data
    impact_type *impacts;    
    unsigned int num_impact;

    // True if we are going to display the gradient grid    
    bool show_grad;
    
    bool show_markers;
    bool show_impacts;
    bool show_map;
    

    // True to display walls
    bool show_walls;
    
    // Coords of goal
    double target_x, target_y;
    
    // True of target is to be followed
    bool target_enabled;
    

    
    // Tweakable values
    double scale1;
    double scale2;
    double scale3;

    
    
    robot_class *robots[MAX_ROBOT];         // All robots
    unsigned int num_robots;                // Robot count


  public:
  
   
    main_class *g_main;
    
};

#endif





